@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpctl.c,v 1.126 2007/08/06 19:16:06 sobrado Exp $ */
+/*	$OpenBSD: bgpctl.c,v 1.132 2008/02/24 21:02:11 claudio Exp $ */
 
 /*
  * Copyright (c) 2003 Henning Brauer <henning@openbsd.org>
@@ -53,8 +53,7 @@ int		 show_neighbor_terse(struct imsg *);
 int		 show_neighbor_msg(struct imsg *, enum neighbor_views);
 void		 print_neighbor_capa_mp_safi(u_int8_t);
 void		 print_neighbor_msgstats(struct peer *);
-void		 print_neighbor_timers(struct peer *);
-void		 print_timer(const char *, time_t, u_int);
+void		 print_timer(const char *, time_t);
 static char	*fmt_timeframe(time_t t);
 static char	*fmt_timeframe_core(time_t t);
 void		 show_fib_head(void);
@@ -67,7 +66,7 @@ void		 show_interface_head(void);
 int		 ift2ifm(int);
 const char *	 get_media_descr(int);
 const char *	 get_linkstate(int, int);
-void		 print_baudrate(u_long);
+void		 print_baudrate(u_int64_t);
 int		 show_interface_msg(struct imsg *);
 void		 show_rib_summary_head(void);
 void		 print_prefix(struct bgpd_addr *, u_int8_t, u_int8_t);
@@ -76,6 +75,8 @@ void		 print_flags(u_int8_t, int);
 int		 show_rib_summary_msg(struct imsg *);
 int		 show_rib_detail_msg(struct imsg *, int);
 void		 show_community(u_char *, u_int16_t);
+const char	*get_ext_subtype(u_int8_t);
+void		 show_ext_community(u_char *, u_int16_t);
 char		*fmt_mem(int64_t);
 int		 show_rib_memory_msg(struct imsg *);
 void		 send_filterset(struct imsgbuf *, struct filter_set_head *);
@@ -196,6 +197,7 @@ main(int argc, char *argv[])
 	case SHOW_NEIGHBOR:
 	case SHOW_NEIGHBOR_TIMERS:
 	case SHOW_NEIGHBOR_TERSE:
+		neighbor.show_timers = (res->action == SHOW_NEIGHBOR_TIMERS);
 		if (res->peeraddr.af || res->peerdesc[0])
 			imsg_compose(ibuf, IMSG_CTL_SHOW_NEIGHBOR, 0, 0, -1,
 			    &neighbor, sizeof(neighbor));
@@ -518,6 +520,7 @@ int
 show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 {
 	struct peer		*p;
+	struct ctl_timer	*t;
 	struct in_addr		 ina;
 	char			 buf[NI_MAXHOST], pbuf[NI_MAXSERV], *s;
 
@@ -582,14 +585,9 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 				printf("    4-byte AS numbers\n");
 		}
 		printf("\n");
-		switch (nv) {
-		case NV_DEFAULT:
-			print_neighbor_msgstats(p);
+		if (nv == NV_TIMERS)
 			break;
-		case NV_TIMERS:
-			print_neighbor_timers(p);
-			break;
-		}
+		print_neighbor_msgstats(p);
 		printf("\n");
 		if (p->state == STATE_IDLE) {
 			static const char	*errstr;
@@ -621,6 +619,11 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 			printf("\n");
 		}
 		break;
+	case IMSG_CTL_SHOW_TIMER:
+		t = imsg->data;
+		if (t->type > 0 && t->type < Timer_Max)
+			print_timer(timernames[t->type], t->val);
+		break;
 	case IMSG_CTL_END:
 		return (1);
 		break;
@@ -679,31 +682,14 @@ print_neighbor_msgstats(struct peer *p)
 }
 
 void
-print_neighbor_timers(struct peer *p)
+print_timer(const char *name, timer_t d)
 {
-	print_timer("IdleHoldTimer:", p->IdleHoldTimer, p->IdleHoldTime);
-	print_timer("ConnectRetryTimer:", p->ConnectRetryTimer,
-	    INTERVAL_CONNECTRETRY);
-	print_timer("HoldTimer:", p->HoldTimer, (u_int)p->holdtime);
-	print_timer("KeepaliveTimer:", p->KeepaliveTimer, (u_int)p->holdtime/3);
-}
-
-void
-print_timer(const char *name, time_t val, u_int interval)
-{
-	int	d;
-
-	d = val - time(NULL);
 	printf("  %-20s ", name);
 
-	if (val == 0)
-		printf("%-20s", "not running");
-	else if (d <= 0)
-		printf("%-20s", "due");
+	if (d <= 0)
+		printf("%-20s\n", "due");
 	else
-		printf("due in %-13s", fmt_timeframe_core(d));
-
-	printf("Interval: %5us\n", interval);
+		printf("due in %-13s\n", fmt_timeframe_core(d));
 }
 
 #define TF_BUFS	8
@@ -724,7 +710,7 @@ fmt_timeframe_core(time_t t)
 	char		*buf;
 	static char	 tfbuf[TF_BUFS][TF_LEN];	/* ring buffer */
 	static int	 idx = 0;
-	unsigned	 sec, min, hrs, day, week;
+	unsigned int	 sec, min, hrs, day, week;
 
 	buf = tfbuf[idx++];
 	if (idx == TF_BUFS)
@@ -972,16 +958,16 @@ get_linkstate(int media_type, int link_state)
 }
 
 void
-print_baudrate(u_long baudrate)
+print_baudrate(u_int64_t baudrate)
 {
 	if (baudrate > IF_Gbps(1))
-		printf("%lu GBit/s", baudrate / IF_Gbps(1));
+		printf("%llu GBit/s", baudrate / IF_Gbps(1));
 	else if (baudrate > IF_Mbps(1))
-		printf("%lu MBit/s", baudrate / IF_Mbps(1));
+		printf("%llu MBit/s", baudrate / IF_Mbps(1));
 	else if (baudrate > IF_Kbps(1))
-		printf("%lu KBit/s", baudrate / IF_Kbps(1));
+		printf("%llu KBit/s", baudrate / IF_Kbps(1));
 	else
-		printf("%lu Bit/s", baudrate);
+		printf("%llu Bit/s", baudrate);
 }
 
 int
@@ -1131,7 +1117,8 @@ show_rib_detail_msg(struct imsg *imsg, int nodescr)
 	struct in_addr		 id;
 	char			*aspath, *s;
 	u_char			*data;
-	u_int16_t		 ilen, alen;
+	u_int32_t		 as;
+	u_int16_t		 ilen, alen, ioff;
 	u_int8_t		 flags, type;
 	time_t			 now;
 
@@ -1173,28 +1160,61 @@ show_rib_detail_msg(struct imsg *imsg, int nodescr)
 	case IMSG_CTL_SHOW_RIB_ATTR:
 		ilen = imsg->hdr.len - IMSG_HEADER_SIZE;
 		if (ilen < 3)
-			break;
+			errx(1, "bad IMSG_CTL_SHOW_RIB_ATTR received");
 		data = imsg->data;
 		flags = data[0];
 		type = data[1];
-		if (type == ATTR_COMMUNITIES) {
-			if (flags & ATTR_EXTLEN) {
-				if (ilen < 4)
-					break;
-				memcpy(&alen, data+2, sizeof(u_int16_t));
-				alen = ntohs(alen);
-				data += 4;
-				ilen -= 4;
-			} else {
-				alen = data[2];
-				data += 3;
-				ilen -= 3;
-			}
-			if (alen != ilen)
-				break;
-			printf("    Community: ");
+
+		/* get the attribute length */
+		if (flags & ATTR_EXTLEN) {
+			if (ilen < 4)
+				errx(1, "bad IMSG_CTL_SHOW_RIB_ATTR received");
+			memcpy(&alen, data+2, sizeof(u_int16_t));
+			alen = ntohs(alen);
+			data += 4;
+			ilen -= 4;
+		} else {
+			alen = data[2];
+			data += 3;
+			ilen -= 3;
+		}
+		/* bad imsg len how can that happen!? */
+		if (alen != ilen)
+			errx(1, "bad IMSG_CTL_SHOW_RIB_ATTR received");
+
+		switch (type) {
+		case ATTR_COMMUNITIES:
+			printf("    Communities: ");
 			show_community(data, alen);
 			printf("\n");
+			break;
+		case ATTR_AGGREGATOR:
+			memcpy(&as, data, sizeof(as));
+			memcpy(&id, data + sizeof(as), sizeof(id));
+			printf("    Aggregator: %s [%s]\n", log_as(as),
+			   inet_ntoa(id));
+			break;
+		case ATTR_ORIGINATOR_ID:
+			memcpy(&id, data, sizeof(id));
+			printf("    Originator Id: %s\n", inet_ntoa(id));
+			break;
+		case ATTR_CLUSTER_LIST:
+			printf("    Cluster ID List:");
+			for (ioff = 0; ioff + sizeof(id) <= ilen;
+			    ioff += sizeof(id)) {
+				memcpy(&id, data + ioff, sizeof(id));
+				printf(" %s", inet_ntoa(id));
+			}
+			printf("\n");
+			break;
+		case ATTR_EXT_COMMUNITIES:
+			printf("    Ext. communities: ");
+			show_ext_community(data, alen);
+			printf("\n");
+			break;
+		default:
+			/* ignore unknown attributes */
+			break;
 		}
 		break;
 	case IMSG_CTL_END:
@@ -1306,6 +1326,78 @@ show_community(u_char *data, u_int16_t len)
 	}
 }
 
+const char *
+get_ext_subtype(u_int8_t type)
+{
+	static char etype[6];
+
+	switch (type) {
+	case EXT_COMMUNITY_ROUTE_TGT:
+		return "rt";	/* route target */
+	case EXT_CUMMUNITY_ROUTE_ORIG:
+		return "soo";	/* source of origin */
+	case EXT_COMMUNITY_OSPF_DOM_ID:
+		return "odi";	/* ospf domain id */
+	case EXT_COMMUNITY_OSPF_RTR_TYPE:
+		return "ort";	/* ospf route type */
+	case EXT_COMMUNITY_OSPF_RTR_ID:
+		return "ori";	/* ospf router id */
+	case EXT_COMMUNITY_BGP_COLLECT:
+		return "bdc";	/* bgp data collection */
+	default:
+		snprintf(etype, sizeof(etype), "[%i]", (int)type);
+		return etype;
+	}
+}
+
+void
+show_ext_community(u_char *data, u_int16_t len)
+{
+	u_int64_t	ext;
+	struct in_addr	ip;
+	u_int32_t	as4, u32;
+	u_int16_t	i, as2, u16;
+	u_int8_t	type, subtype;
+
+	if (len & 0x7)
+		return;
+
+	for (i = 0; i < len; i += 8) {
+		type = data[i];
+		subtype = data[i + 1];
+
+		switch (type & EXT_COMMUNITY_VALUE) {
+		case EXT_COMMUNITY_TWO_AS:
+			memcpy(&as2, data + i + 2, sizeof(as2));
+			memcpy(&u32, data + i + 4, sizeof(u32));
+			printf("%s %hu:%u", get_ext_subtype(subtype), as2, u32);
+			break;
+		case EXT_COMMUNITY_IPV4:
+			memcpy(&ip, data + i + 2, sizeof(ip));
+			memcpy(&u16, data + i + 6, sizeof(u16));
+			printf("%s %s:%hu", get_ext_subtype(subtype),
+			    inet_ntoa(ip), u16);
+			break;
+		case EXT_COMMUNITY_FOUR_AS:
+			memcpy(&as4, data + i + 2, sizeof(as4));
+			memcpy(&u16, data + i + 6, sizeof(u16));
+			printf("%s %s:%hu", get_ext_subtype(subtype),
+			    log_as(as4), u16);
+			break;
+		case EXT_COMMUNITY_OPAQUE:
+			memcpy(&ext, data + i, sizeof(ext));
+			ext = betoh64(ext) & 0xffffffffffffLL;
+			printf("%s 0x%llx", get_ext_subtype(subtype), ext); 
+			break;
+		default:
+			memcpy(&ext, data + i, sizeof(ext));
+			printf("0x%llx", betoh64(ext)); 
+		}
+		if (i + 8 < len)
+			printf(", ");
+	}
+}
+
 void
 send_filterset(struct imsgbuf *i, struct filter_set_head *set)
 {
